@@ -134,6 +134,7 @@ typeVars <- list(
   "authorizedApplication.authorizedPartI.productRoleGroupInfos.products.part1MedicinalProductRoleMscInfos.memberStateConcernedInfo.trialRecruitmentPeriod.fromDate" = "ctrDate",
   "authorizedApplication.authorizedPartI.productRoleGroupInfos.products.part1MedicinalProductRoleMscInfos.memberStateConcernedInfo.trialRecruitmentPeriod.recruitmentEndDate" = "ctrDate",
   "authorizedApplication.authorizedPartI.productRoleGroupInfos.products.part1MedicinalProductRoleMscInfos.memberStateConcernedInfo.trialRecruitmentPeriod.recruitmentStartDate" = "ctrDate",
+  "authorizedApplication.authorizedPartI.productRoleGroupInfos.products.part1MedicinalProductRoleMscInfos.memberStateConcernedInfo.trialRestartDate" = "ctrDate",
   "authorizedApplication.authorizedPartI.products.part1MedicinalProductRoleMscInfos.memberStateConcernedInfo.activeTrialPeriod.fromDate" = "ctrDate",
   "authorizedApplication.authorizedPartI.products.part1MedicinalProductRoleMscInfos.memberStateConcernedInfo.activeTrialPeriod.trialEndDate" = "ctrDate",
   "authorizedApplication.authorizedPartI.products.part1MedicinalProductRoleMscInfos.memberStateConcernedInfo.activeTrialPeriod.trialStartDate" = "ctrDate",
@@ -148,6 +149,7 @@ typeVars <- list(
   "authorizedApplication.authorizedPartI.products.part1MedicinalProductRoleMscInfos.memberStateConcernedInfo.trialRecruitmentPeriod.fromDate" = "ctrDate",
   "authorizedApplication.authorizedPartI.products.part1MedicinalProductRoleMscInfos.memberStateConcernedInfo.trialRecruitmentPeriod.recruitmentEndDate" = "ctrDate",
   "authorizedApplication.authorizedPartI.products.part1MedicinalProductRoleMscInfos.memberStateConcernedInfo.trialRecruitmentPeriod.recruitmentStartDate" = "ctrDate",
+  "authorizedApplication.authorizedPartI.products.part1MedicinalProductRoleMscInfos.memberStateConcernedInfo.trialRestartDate" = "ctrDate",
   "authorizedApplication.authorizedPartI.trialDetails.associatedClinicalTrials.associatedCtDocs.submissionDate" = "ctrDate",
   "authorizedApplication.authorizedPartI.trialDetails.trialInformation.trialDuration.estimatedEndDate" = "ctrDate",
   "authorizedApplication.authorizedPartI.trialDetails.trialInformation.trialDuration.estimatedGlobalEndDate" = "ctrDate",
@@ -530,17 +532,17 @@ typeVars <- list(
   "provided_document_section.provided_document.document_has_sap" = "ctrFalseTrue",
   #
   # - CTGOV2
-  "documentSection.largeDocumentModule.largeDocs.hasIcf"= "ctrFalseTrue",
-  "documentSection.largeDocumentModule.largeDocs.hasProtocol"= "ctrFalseTrue",
-  "documentSection.largeDocumentModule.largeDocs.hasSap"= "ctrFalseTrue",
+  "documentSection.largeDocumentModule.largeDocs.hasIcf"      = "ctrFalseTrue",
+  "documentSection.largeDocumentModule.largeDocs.hasProtocol" = "ctrFalseTrue",
+  "documentSection.largeDocumentModule.largeDocs.hasSap"      = "ctrFalseTrue",
   "hasResults" = "ctrFalseTrue",
-  "protocolSection.eligibilityModule.healthyVolunteers"= "ctrFalseTrue",
-  "protocolSection.oversightModule.isFdaRegulatedDevice"= "ctrFalseTrue",
-  "protocolSection.oversightModule.isFdaRegulatedDrug"= "ctrFalseTrue",
-  "protocolSection.oversightModule.oversightHasDmc"= "ctrFalseTrue",
-  "protocolSection.statusModule.expandedAccessInfo.hasExpandedAccess"= "ctrFalseTrue",
-  "resultsSection.moreInfoModule.certainAgreement.piSponsorEmployee"= "ctrFalseTrue",
-  "resultsSection.moreInfoModule.certainAgreement.restrictiveAgreement"= "ctrFalseTrue",
+  "protocolSection.eligibilityModule.healthyVolunteers"  = "ctrFalseTrue",
+  "protocolSection.oversightModule.isFdaRegulatedDevice" = "ctrFalseTrue",
+  "protocolSection.oversightModule.isFdaRegulatedDrug"   = "ctrFalseTrue",
+  "protocolSection.oversightModule.oversightHasDmc"      = "ctrFalseTrue",
+  "protocolSection.statusModule.expandedAccessInfo.hasExpandedAccess"   = "ctrFalseTrue",
+  "resultsSection.moreInfoModule.certainAgreement.piSponsorEmployee"    = "ctrFalseTrue",
+  "resultsSection.moreInfoModule.certainAgreement.restrictiveAgreement" = "ctrFalseTrue",
   #
   # - ISRCTN
   "trialDescription.acknowledgment" = "ctrFalseTrue",
